@@ -20,6 +20,16 @@ class TreebeardEnv(BaseModel):
     email: Optional[str] = None  # Not present at build time
     api_key: Optional[str] = None  # Not present at build time
 
+    def __str__(self) -> str:
+        dict_obj = self.dict()
+        if self.api_key:
+            unredacted_length = 4
+            dict_obj["api_key"] = (
+                "x" * (len(self.api_key) - unredacted_length)
+                + self.api_key[-unredacted_length:]
+            )
+        return str(dict_obj)
+
 
 class TreebeardConfig(BaseModel):
     notebook: str = "main.ipynb"
