@@ -2,6 +2,7 @@ import glob
 import json
 import os
 import os.path
+import pprint
 import sys
 import tarfile
 import tempfile
@@ -10,25 +11,29 @@ from datetime import datetime
 from typing import Any, List
 
 import click
+import docker  # type: ignore
 import requests
 from dateutil import parser
 from halo import Halo  # type: ignore
 from humanfriendly import format_size, parse_size  # type: ignore
 from timeago import format as timeago_format  # type: ignore
 
-from treebeard.conf import notebooks_endpoint
-from treebeard.helper import (
-    get_config_path,
-    get_treebeard_env,
+from treebeard.conf import (
+    config_path,
     get_time,
+    notebooks_endpoint,
+    treebeard_config,
+    treebeard_env,
     validate_notebook_directory,
-    get_treebeard_config,
 )
+from treebeard.helper import CliContext
+from treebeard.notebooks.types import Run
+from treebeard.secrets.helper import get_secrets_archive
 
-config_path = get_config_path()
+pp = pprint.PrettyPrinter(indent=2)
 
-treebeard_env = get_treebeard_env()
-notebook_id = treebeard_env["notebook_id"]
+notebook_id = treebeard_env.notebook_id
+project_id = treebeard_env.project_id
 
 
 @click.command()
@@ -41,29 +46,36 @@ notebook_id = treebeard_env["notebook_id"]
 @click.option(
     "watch", "--watch", is_flag=True, help="Run and check completed build status"
 )
+@click.option(
+    "local", "--local", is_flag=True, help="Build image with local docker installation",
+)
 @click.option(
     "-i",
     "--ignore",
     help="Don't submit unneeded virtual envs and large files",
     multiple=True,
 )
-def run(t: str, watch: bool, ignore: List[str]):
+@click.pass_obj
+def run(cli_context: CliContext, t: str, watch: bool, ignore: List[str], local: bool):
     """
     Run a notebook and optionally schedule it to run periodically
     """
 
-    validate_notebook_directory()
+    validate_notebook_directory(treebeard_env, treebeard_config)
 
     params = {}
     if t:
         params["schedule"] = t
 
-    spinner: Any = Halo(text="🌲  zipping directory\n", spinner="dots")
+    spinner: Any = Halo(text="🌲  Compressing Repo\n", spinner="dots")
     spinner.start()
 
-    treebeard_config = get_treebeard_config()
     if treebeard_config:
-        ignore += treebeard_config.ignore + treebeard_config.secret
+        ignore += (
+            treebeard_config.ignore
+            + treebeard_config.secret
+            + treebeard_config.output_dirs
+        )
 
     # Create a temporary file for the compressed directory
     # compressed file accessible at f.name
@@ -75,8 +87,11 @@ def run(t: str, watch: bool, ignore: List[str]):
     #     .splitlines()
     # )
 
-    with tempfile.NamedTemporaryFile("wb", suffix=".tar.gz", delete=False) as f:
-        with tarfile.open(fileobj=f, mode="w:gz") as tar:
+    with tempfile.NamedTemporaryFile(
+        "wb", suffix=".tar.gz", delete=False
+    ) as src_archive:
+        click.echo("\n")
+        with tarfile.open(fileobj=src_archive, mode="w:gz") as tar:
 
             def zip_filter(info: tarfile.TarInfo):
                 for ignored in ignore:
@@ -85,15 +100,14 @@ def run(t: str, watch: bool, ignore: List[str]):
 
                 # if len(git_files) > 0 and info.name not in git_files:
                 #     return None
-                click.echo(f"Including {info.name}")
+                click.echo(f"  Including {info.name}")
                 return info
 
             tar.add(
                 os.getcwd(), arcname=os.path.basename(os.path.sep), filter=zip_filter
             )
             tar.add(config_path, arcname=os.path.basename(config_path))
-    spinner.text = "🌲  submitting notebook to runner\n"
-    size = os.path.getsize(f.name)
+    size = os.path.getsize(src_archive.name)
     max_upload_size = "100MB"
     if size > parse_size(max_upload_size):
         click.echo(
@@ -108,11 +122,55 @@ def run(t: str, watch: bool, ignore: List[str]):
         )
         quit(1)
 
+    if local:
+        spinner.stop()
+        spinner: Any = Halo(text="🌲  Compressing Secrets\n", spinner="dots")
+        # spinner.start()
+        # os.chdir(tempfile.gettempdir())
+        client: Any = docker.from_env()  # type: ignore
+        builder_image_name = "gcr.io/treebeard-259315/image-builder"
+        # client.images.pull(builder_image_name)
+        secrets_archive = get_secrets_archive()
+
+        build_tag = str(time.mktime(datetime.today().timetuple()))
+        env_vars = {
+            "TREEBEARD_PROJECT_ID": project_id,
+            "TREEBEARD_NOTEBOOK_ID": notebook_id,
+            "TREEBEARD_REPO_URL": f"file://{src_archive.name}",
+            "TREEBEARD_SECRETS_URL": f"file://{secrets_archive.name}",
+            "TREEBEARD_BUILD_TAG": build_tag,
+        }
+        if cli_context.debug:
+            click.echo(f"Env vars:\n{pp.pformat(env_vars)}")
+
+        click.echo("🌲  Building Image Locally\n")
+        docker_socket = "/var/run/docker.sock"
+        repo_image_name = (
+            f"gcr.io/treebeard-259315/projects/{project_id}/{notebook_id}:{build_tag}"
+        )
+        click.echo(f"Building {repo_image_name}")
+
+        container = client.containers.run(
+            builder_image_name,
+            "python -m treebeard.buildtime.run_repo",
+            environment=env_vars,
+            volumes={
+                src_archive.name: {"bind": src_archive.name, "mode": "ro"},
+                secrets_archive.name: {"bind": secrets_archive.name, "mode": "ro"},
+                docker_socket: {"bind": docker_socket, "mode": "rw"},
+            },
+            detach=True,
+        )
+
+        [click.echo(line, nl=False) for line in container.logs(stream=True)]
+        sys.exit(0)
+
+    spinner.text = "🌲  submitting notebook to runner\n"
     response = requests.post(
         notebooks_endpoint,
-        files={"repo": open(f.name, "rb")},
+        files={"repo": open(src_archive.name, "rb")},
         params=params,
-        headers=treebeard_env,
+        headers=treebeard_env.dict(),
     )
 
     if response.status_code != 200:
@@ -132,7 +190,7 @@ def run(t: str, watch: bool, ignore: List[str]):
         build_result = None
         while not build_result:
             time.sleep(5)
-            response = requests.get(notebooks_endpoint, headers=treebeard_env)
+            response = requests.get(notebooks_endpoint, headers=treebeard_env.dict())
             json_data = json.loads(response.text)
             status = json_data["runs"][-1]["status"]
             click.echo(f"{get_time()} Build status: {status}")
@@ -149,7 +207,7 @@ def run(t: str, watch: bool, ignore: List[str]):
 @click.command()
 def cancel():
     """Cancels the current notebook build and schedule"""
-    validate_notebook_directory()
+    validate_notebook_directory(treebeard_env, treebeard_config)
     spinner: Any = Halo(text="cancelling", spinner="dots")
     click.echo(f"🌲  Cancelling {notebook_id}")
     spinner.start()
@@ -161,8 +219,8 @@ def cancel():
 @click.command()
 def status():
     """Show the status of the current notebook"""
-    validate_notebook_directory()
-    response = requests.get(notebooks_endpoint, headers=treebeard_env)
+    validate_notebook_directory(treebeard_env, treebeard_config)
+    response = requests.get(notebooks_endpoint, headers=treebeard_env.dict())
     if response.status_code != 200:
         raise click.ClickException(f"Request failed: {response.text}")
 
@@ -174,7 +232,7 @@ def status():
         quit(1)
     click.echo("🌲  Recent runs:\n")
 
-    max_results = 10
+    max_results = 5
     status_emoji = {
         "SUCCESS": "✅",
         "QUEUED": "💤",
@@ -184,11 +242,17 @@ def status():
         "CANCELLED": "🛑",
     }
 
-    for run in json_data["runs"][-max_results:]:
+    runs: List[Run] = [Run.parse_obj(run) for run in json_data["runs"][-max_results:]]  # type: ignore
+    for run in runs:
         now = parser.isoparse(datetime.utcnow().isoformat() + "Z")
-        start_time = parser.isoparse(run["start_time"])
+        start_time = parser.isoparse(run.start_time)
         time_string: str = timeago_format(start_time, now=now)
-        click.echo(f"{status_emoji[run['status']]}  {time_string} {run['url']}")
+
+        mechanism: str = run.trigger["mechanism"]
+        ran_via = "" if len(mechanism) == 0 else f"via {mechanism}"
+        click.echo(
+            f"  {status_emoji[run.status]}  {time_string} {ran_via} -- {run.url}"
+        )
 
     if "schedule" in json_data:
         click.echo(f"\n📅  Schedule: {json_data['schedule']}")
